//using namespace std;
#include "GBS_Control.h"
//#include <avr/pgmspace.h>
//#include "I2CBitBanger.h"
//#include "StartArray.h"
//#include "ProgramArray240p.h"
//#include "ProgramArray288p50.h"
//#include "ProgramArray480i.h"
//#include "ProgramArray576i50.h"
//#include "NECIRReceiver.h"
//#include "RemoteControlButtonValues.h"
//#include <EEPROM.h>

// I2C stuff
#define GBS_I2C_ADDRESS 0x17 // 0x2E
I2CBitBanger i2cObj(GBS_I2C_ADDRESS);

// remote control stuff
#define GBS_REMOTE_DSPARK_GPIO_PIN 11
//NECIRReceiver gbsRemoteControl(GBS_REMOTE_DSPARK_GPIO_PIN);

// persitent storage related items
#define VALID_BANK 0x7e
#define SCALING_SEGMENT 0x03
#define HRST_LOW  0x01
#define HRST_HIGH 0x02
#define HBST_LOW  0x04
#define HBST_HIGH 0x05
#define HBSP_LOW  0x05
#define HBSP_HIGH 0x06
#define HSCALE_LOW  0x16
#define HSCALE_HIGH 0x17
#define VRST_LOW  0x02
#define VRST_HIGH 0x03
#define VBST_LOW  0x07
#define VBST_HIGH 0x08
#define VBSP_LOW  0x08
#define VBSP_HIGH 0x09

struct SettingsBank {
  /*
   Settings banks are stored in onboard EEPROM.
   Each bank contains 10 bytes of data
   One bank consists of the following:
     byte     value
     0        validity (byte value VALID_BANK (i.e. 0x7e) if bank contains valid data.  any other value indicates the bank should be ignored)
     1        horizontal shift low
     2        horizontal shift med
     3        horizontal shift high
     4        horizontal scale low
     5        horizontal scale high
     6        vertical shift low
     7        veritcal shift med
     8        vertical shift high
     9        reserved (value currently undefined)
 
  uint8_t valid;
  uint8_t hShiftLow;
  uint8_t hShiftMed;
  uint8_t hShiftHigh;
  uint8_t hScaleLow;
  uint8_t hScaleHigh;
  uint8_t vShiftLow;
  uint8_t vShiftMed;
  uint8_t vShiftHigh;
  uint8_t reserved;
  */
  uint8_t data[10];
};


bool writeOneByte(uint8_t slaveRegister, uint8_t value)
{
  return writeBytes(slaveRegister, &value, 1); 
}


bool writeBytes(uint8_t slaveAddress, uint8_t slaveRegister, uint8_t* values, uint8_t numValues)
{
  /*
   Write to Consecutive Control Registers:
    -Start Signal 
    -Slave Address Byte (R/W Bit = Low) 
    -Base Address Byte 
    -Data Byte to Base Address 
    -Data Byte to (Base Address + 1) 
    -Data Byte to (Base Address + 2) 
    -Data Byte to (Base Address + 3) 
    .................. 
    -Stop Signal
  */
 
  //i2cObj.setSlaveAddress(slaveAddress);
  
  i2cObj.addByteForTransmission(slaveRegister);
  
  i2cObj.addBytesForTransmission(values, numValues);
  
  if(!i2cObj.transmitData())
  {    
    return false;
  }
 
  return true;
}


bool writeBytes(uint8_t slaveRegister, uint8_t* values, int numValues)
{
  return writeBytes(GBS_I2C_ADDRESS, slaveRegister, values, numValues);
}


bool writeStartArray()
{
  for(int z = 0; z < ((sizeof(startArray)/2) - 1520); z++)
  {
    writeOneByte(pgm_read_byte(startArray + (z*2)), pgm_read_byte(startArray + (z*2) + 1));
    _delay_ms(1);
  }
    
  return true;
}



bool writeProgramArray(const uint8_t* programArray)
{ 
  for(int y = 0; y < 6; y++)
  { 
    writeOneByte(0xF0, (uint8_t)y );
    _delay_ms(1);
 
    for(int z = 0; z < 15; z++)
    {
      uint8_t bank[16];
      for(int w = 0; w < 16; w++)
      {
        bank[w] = pgm_read_byte(programArray + (y*256 + z*16 + w));
      }
      
      writeBytes(z*16, bank, 16);
      
      _delay_ms(1);
    }
    
  }
  
  return true;
}


int readFromRegister(uint8_t segment, uint8_t reg, int bytesToRead, uint8_t* output) {
  
  // go to the appropriate segment
  if(!writeOneByte(0xF0, segment)) {
    return 0;
  }
  
  return readFromRegister(reg, bytesToRead, output);
}

int readFromRegister(uint8_t reg, int bytesToRead, uint8_t* output) {
    
  // go to the appropriate register
  i2cObj.addByteForTransmission(reg);
  i2cObj.transmitData();
  
  return i2cObj.recvData(bytesToRead, output);
}


// Macros for a "Resolution" switch which allows for toggling between
// 240p (1) and 480i (0)
//
// NOTE: The switch schematic below has an RC debouncer connected to it for stability
//
// The presence of this switch is optional.  The 50ms delay in the loop could be enough to mask the bounce of your switch.
// WARNING: Depending on switch/button connection, you may need to enable internal pull up resistors for pins.  This would 
//
//                                  
// Digispark VCC -----/\/\/\/----------| SPDT switch |------------/\/\/\/-----GND
//                       R1                  |                      R2        |
//                                           |                                |
//                                           |----------------------||--------|           
//                                           |                      C1
//                                           |
//                                  Digispark Pro Input
//                                     GPIO 5 (PA7)
//
// R1, R2, and C1 values depend on your switch's debounce time

#define RESOLUTION_SWITCH_DDR  DDRB
#define RESOLUTION_SWITCH_PIN  PINB
#define RESOLUTION_SWITCH_PORT PORTB
#define RESOLUTION_SWITCH_BIT  0

// This variable holds the last known state of the Resolution switch
// true = 240p, false = 480i
bool lastKnownResolutionSwitchState = true;


// Digispark onboard LED macros
#define LED_DDR DDRB
#define LED_PORT PORTB
#define LED_BIT 13

void returnToDefaultSettings() {
  writeProgramArray(programArray576i50);
  //writeProgramArray(programArray480i); //
  LED_PORT &= ~(1<<LED_BIT); // turn off LED
}

  
int
main (void)
{


  _delay_ms(100); // delay 

  writeStartArray();
  writeProgramArray(programArray288p50);

  RESOLUTION_SWITCH_DDR &= ~(1<<RESOLUTION_SWITCH_BIT); // Set the resolution switch to an input
  RESOLUTION_SWITCH_PORT &= ~(1<<RESOLUTION_SWITCH_BIT); // Turn off internal pullup (we already have resistors in the debounce circuit).  PULL UPs MAY NEED TO BE ENABLED FOR YOUR CIRCUIT

  lastKnownResolutionSwitchState = true;

  while(1) {
     _delay_ms(500); // delay 
     // Read the state of the Resolution switch
     bool currentResolutionSwitchState = ((RESOLUTION_SWITCH_PIN & (1<<RESOLUTION_SWITCH_BIT)) != 0);
  
     if(currentResolutionSwitchState != lastKnownResolutionSwitchState)
     {
       // The switch state has been flipped
       // Write the appropriate register values to the scaler chip
       
       if(currentResolutionSwitchState == true) {
         // Write the 240p register values to the scaler chip
         writeProgramArray(programArray288p50);
         //LED_PORT |= (1<<LED_BIT); // turn on LED
       } else {
         // Write the 480i register values to the scaler chip
         //writeProgramArray(programArray480i); // original set values by dooklink
         writeProgramArray(programArray576i50); //
         //LED_PORT &= ~(1<<LED_BIT); // turn off LE
       }
       
       lastKnownResolutionSwitchState = currentResolutionSwitchState;
     }
  }     
   
}
